from setuptools import setup

setup(
    name='blender-plots',
    version='1.0.12',
    author='Linus Härenstam-Nielsen',
    description='Plotting API for Blender',
    url='https://github.com/Linusnie/blender-plots',
    python_requires='>=3.10',
    packages=['blender_plots'],
)